﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CamFX : MonoBehaviour
{

    [SerializeField] private Camera _cam;
    public Camera cam{get{return _cam;}}

    public GameObject focus;


    private float _lookahead;


    public Config currentConfig;
    public Context[] contexts;

    public Shake screenShake;

    [System.Serializable]
    public class Shake{
        private Camera cam;
        public float shakeStrength=20f;
        public float smoothness=5f;
        public float decayStrength=10f;

        public float shakeMax=20f;

        private float _shaking;

        public void Init(CamFX cam_){
            cam=cam_.cam;
        }

        public void AddShake(float amount_){
            _shaking+=amount_;
        }

        public void Update(){
            _shaking-=Time.deltaTime*decayStrength;
            if(_shaking<0){_shaking=0;}
            if(_shaking>shakeMax){_shaking=shakeMax;}
            cam.transform.rotation=Quaternion.Euler(
                //cam.transform.rotation.eulerAngles.x+shakeStrength*_shaking*Mathf.PerlinNoise(Time.fixedTime,_shaking/smoothness),
                cam.transform.rotation.eulerAngles.x,
                cam.transform.rotation.eulerAngles.y+shakeStrength*_shaking*Mathf.PerlinNoise(Time.fixedTime+3.5261f,_shaking/smoothness),
                cam.transform.rotation.eulerAngles.z);
        }
    }

    [System.Serializable]
    public class Context{
        public ContextType type;
        public Config config;
    }

    public enum ContextType{
        ON_FOOT,
        ON_STAR
    }

    [System.Serializable]
    public class Config{
        public float distance;
        public float overhead;
        public float overshoulder;
        public float zoom;
        public float tiltDown;
        public float horizontalLookahead;
        public float lookaheadSpeed;

        public float lookaheadDampen{get{if(_lookaheadDampen==0){return 1;}else{return _lookaheadDampen;}}}
        public float _lookaheadDampen;

        public float lookaheadDelay;
        public float rotateSmoothing;
        public float positionSmoothing;
    }

    private Vector3 focusPos{get{
        if(!focus){return Vector3.zero;}
        return focus.transform.position;
        }
    }
    private Vector3 orientDirection{get{
        return Vector3.ProjectOnPlane(GameManager.Instance.NextHolePos-focus.transform.position,Vector3.up).normalized;
    }}

    public void SetFocus(GameObject focus_){
        focus=focus_;
        if(focus==null){return;}
    }
    // Start is called before the first frame update
    void Start()
    {
        screenShake.Init(this);
        SetFocus(focus);
    }

    void LateUpdate(){
        screenShake.Update();
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        updateTransform();
        updateCamTransform();
    }

    public void AddScreenShake(float val_){
        screenShake.AddShake(val_);
    }

    public void ChangeContext(ContextType type_){
        foreach(Context c_ in contexts){
            if(c_.type==type_){
                currentConfig=c_.config;
                return;
            }
        }
    }

    

    private void updateTransform(){
        Vector3 setPos_=Vector3.zero;
        setPos_=-orientDirection*currentConfig.distance;
        setPos_=setPos_+focus.transform.position+currentConfig.overhead*Vector3.up+currentConfig.overshoulder*Vector3.right;

        transform.position=setPos_;

        Quaternion prevRot_=transform.rotation;

        transform.rotation=Quaternion.LookRotation(orientDirection,Vector3.up);
        transform.Rotate(Vector3.right*currentConfig.tiltDown);

    }

    private void updateCamTransform(){
        cam.transform.position=Vector3.Lerp(cam.transform.position,this.transform.position,Time.deltaTime*currentConfig.positionSmoothing);
        cam.transform.rotation=Quaternion.Lerp(cam.transform.rotation,this.transform.rotation,Time.deltaTime*currentConfig.rotateSmoothing);
    }

}
