﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using TMPro;
using UnityEngine.SceneManagement;

public class GameManager : MonoBehaviour
{

    public static GameManager Instance;


    public static GameType CoursesToPlay;

    public enum GameType{
        Front9,
        Back9
    }

    public bool gameOver;

    public GameObject gameOverScreen;

    public BallContext ball;
    public ShotContext shot;
    public WindContext wind;

    public CourseContext course;

    public CourseContext front9,back9;

    public MiniSound windySound,bootSound,strokeSound,holeSound,holeInOneSound,ballHitGround,ballHitSand,ballHitWater;


    public bool OnGreen{get{return MockupManager.Instance.ballState==MockupManager.BallState.Green;}}
    public Vector3 BallVel{get{return ball.RB.velocity;}}
    public bool BallStopped{get{return BallVel.magnitude<.15f;}}
    public bool BallMidair{get{return MockupManager.Instance.ballState==MockupManager.BallState.Airborne;}}
    public bool BallInWater{get{return MockupManager.Instance.ballState==MockupManager.BallState.Water;}}
    public Vector3 NextHolePos{get{return course.currentHole.flag.transform.position;}}
    public Vector3 BallPos{get{return ball.ball.transform.position;}}
    public Vector3 WindForce{get{return wind.WindForce;}}

    public GameObject CurrentHole{get{return course.currentHole.hole.gameObject;}}

    // Start is called before the first frame update
    void Awake()
    {
        Instance=this;

        ball.init();
        shot.init();
        wind.init();

        if(CoursesToPlay==GameType.Front9){
            course=front9;
        }else{
            course=back9;
        }

        course.init();

        windySound.init(this);
        bootSound.init(this);
        strokeSound.init(this);
        holeSound.init(this);
        holeInOneSound.init(this);
        ballHitGround.init(this);
        ballHitSand.init(this);
        ballHitWater.init(this);

    }

    void Start(){
        
        course.teeOff();
    }

    // Update is called once per frame
    void Update()
    {
        if(gameOver){

            return;
        }
        shot.update();
        wind.update();
    }

    void FixedUpdate(){
        ball.onFixedMove();
    }

    public void completeGame(){
        gameOver=true;
        gameOverScreen.SetActive(true);
    }


    public void restart(){
        SceneManager.LoadScene("Menu");
    }


    public static LayerMask GetCollisionMaskOf(GameObject go)
    {
        int myLayer = go.layer;
        int layerMask = 0;
        for (int i = 0; i < 32; i++)
        {
            if (!Physics.GetIgnoreLayerCollision(myLayer, i))
            {
                layerMask = layerMask | 1 << i;
            }
        }
        return layerMask;
    }
}

[System.Serializable]
public class BallContext{
    public GameObject ball;


    private Ball ballComponent;

    private Rigidbody rb;

    public Rigidbody RB{get{return rb;}}
    public void init(){
        rb=ball.GetComponent<Rigidbody>();
        ballComponent=ball.GetComponent<Ball>();
    }

    public void onFixedMove(){
        Vector3 nf_=-rb.velocity.normalized*7;
        switch(MockupManager.Instance.ballState){
            case MockupManager.BallState.Green:
            rb.AddForce(nf_*.25f);
            break;
            case MockupManager.BallState.Rough:
            rb.AddForce(nf_*.9f);
            break;
            case MockupManager.BallState.Fairway:
            rb.AddForce(nf_*.4f);
            break;
            case MockupManager.BallState.Sand:
            rb.velocity=.5f*rb.velocity;
            break;
            case MockupManager.BallState.Water:
            rb.velocity=.5f*rb.velocity;
            GameManager.Instance.shot.revert();
            break;
            case MockupManager.BallState.Airborne:
            rb.AddForce(GameManager.Instance.WindForce);
            break;
        }
    }

    public void shoot(Vector3 force_){
        rb.AddForce(force_,ForceMode.Impulse);
    }

    public void spawnGolfer(){
        ballComponent.emitGolfer();
    }

    public void spawnBoot(){
        ballComponent.emitBoot();
    }

}

[System.Serializable]
public class ShotContext{
    public List<Vector3> shotPos;

    public int strokes;

    public Vector3 minimum;
    public Vector3 maximum;

    public float shotPlanningTime=2f;

    public Vector3 plannedShot;

    public TextMeshProUGUI strokeCount;
    public GameObject golferCursor;

    public float shotPowerIndicatedScale=.3f;

    public float spawnGolferTiming=.5f;

    public float MaxShotPower{get{return maximum.magnitude*shotPowerIndicatedScale;}}

    private bool planningShot;
    private float planningShotTimer;

    private float firedShotTimer;

    private bool spawnedGolfer;

    private bool reverted;

    public void init(){
        plannedShot=new Vector3();
        shotPos=new List<Vector3>();
    }

    public void update(){
        if(firedShotTimer>1f && GameManager.Instance.BallStopped){
            if(planningShot){
                planningShotTimer+=Time.deltaTime;
                if(planningShotTimer>=shotPlanningTime){
                    planningShotTimer=0;
                    shoot();
                }
                if(planningShotTimer>spawnGolferTiming && !spawnedGolfer){
                    GameManager.Instance.ball.spawnGolfer();
                    spawnedGolfer=true;
                }
            }else{
                planshot();
            }
            
        }
        firedShotTimer+=Time.deltaTime;
    }

    private void addStroke(){
        strokes++;
        GameManager.Instance.course.addStroke();
        strokeCount.text=strokes+"";
    }
    public void revert(){
        if(reverted){return;}
        addStroke();
        GameManager.Instance.ballHitWater.play();
        GameManager.Instance.ball.ball.transform.position=shotPos[shotPos.Count-1];
        MockupManager.Instance.ballState=MockupManager.BallState.Rough;
        GameManager.Instance.ball.RB.velocity=Vector3.zero;
        reverted=true;
    }

    private void planshot(){
        if(GameManager.Instance.BallInWater){return;}
        plannedShot.Set(Random.Range(minimum.x,maximum.x),Random.Range(minimum.y,maximum.y),Random.Range(minimum.z,maximum.z));
        if(GameManager.Instance.OnGreen){
            plannedShot=Vector3.ProjectOnPlane(plannedShot,Vector3.up);
        }
        planningShot=true;
        spawnedGolfer=false;
        shotPos.Add(GameManager.Instance.BallPos);
        indicateShot();
    }

    private void indicateShot(){
        Debug.Log("Setting golfer cursor active");
        golferCursor.SetActive(true);
        indicatePos(Vector3.ProjectOnPlane(GameManager.Instance.BallPos-plannedShot*shotPowerIndicatedScale,Vector3.up));
    }

    private void indicatePos(Vector3 pos_){
        GameObject cursor=golferCursor;
        cursor.transform.position=new Vector3(pos_.x,1,pos_.z);

        if(Vector3.Distance(cursor.transform.position,GameManager.Instance.BallPos)>MaxShotPower){
            Vector3 pos2_=(cursor.transform.position-GameManager.Instance.BallPos).normalized*MaxShotPower+GameManager.Instance.BallPos;
            cursor.transform.position=new Vector3(pos2_.x,1,pos2_.z);
        }

        cursor.transform.LookAt(GameManager.Instance.ball.ball.transform,Vector3.up);
        Vector3 e_ = cursor.transform.rotation.eulerAngles;
        cursor.transform.rotation=Quaternion.Euler(0,e_.y,0);
    }

    private void shoot(){
        GameManager.Instance.ball.shoot(plannedShot);
        planningShot=false;
        addStroke();
        golferCursor.SetActive(false);
        firedShotTimer=0;
        Debug.Log("we shot, golfer cursor inactive");
        GameManager.Instance.strokeSound.play();
        reverted=false;
    }
}

[System.Serializable]
public class CourseContext{
    public List<HoleContext> holes;
    public int currentHoleIndex;

    private int currentHoleScore;

    public HoleContext currentHole{get{
        if(currentHoleIndex>=holes.Count){
            return holes[holes.Count-1];
        }
        return holes[currentHoleIndex];
        }}

    public void init(){
        
        foreach(HoleContext h_ in holes){
            h_.init();
        }
        
    }

    public void addStroke(){
        currentHoleScore++;
    }

    public void teeOff(){
        currentHole.teeOff();
    }

    public void advance(){
        
        GameManager.Instance.holeSound.play();

        if(currentHoleScore<=1){
            GameManager.Instance.holeInOneSound.play();
        }

        currentHoleIndex++;
        currentHoleScore=0;
        if(currentHoleIndex>=holes.Count){
            Debug.Log("completed game");
            GameManager.Instance.completeGame();
            return;
        }
        teeOff();
    }
}

[System.Serializable]
public class HoleContext{
    public GameObject flag;
    public GameObject tee;

    [HideInInspector]
    public Collider hole;

    public void init(){
        hole=flag.GetComponentInChildren<SphereCollider>();
    }

    public void teeOff(){
        GameManager.Instance.ball.ball.transform.position=tee.transform.position;
        GameManager.Instance.ball.RB.velocity=Vector3.zero;
        MockupManager.Instance.ballState=MockupManager.BallState.Fairway;
    }

    public void inHole(){
        if(GameManager.Instance.gameOver){return;}
        GameManager.Instance.course.advance();
    }
}

[System.Serializable]
public class WindContext{
    public Vector3 wind;

    public Vector3 kick;
    public float windStrength=3;
    public float kickStrength=3;
    public float maxWind=10;

    public GameObject worldPlane;

    public GameObject cursor;

    public ParticleSystem windParts;

    public TextMeshProUGUI bootCountText;
    public Image bootRechargeFill;

    public int boots;
    private float bootRecharge;

    public Vector3 WindForce{get{return Vector3.ProjectOnPlane(wind*windStrength,Vector3.up);}}
    public Vector3 KickForce{get{return Vector3.ProjectOnPlane(kick*.1f*kickStrength,Vector3.up);}}

    

    public void init(){
        wind=Vector3.forward;
        kick=Vector3.forward;

    }

    public void update(){
        if(GameManager.Instance.gameOver){return;}

        bootRecharge+=Time.deltaTime/8f;
        if(GameManager.Instance.OnGreen){
            bootRecharge+=Time.deltaTime/8f;
        }
        

        if(bootRecharge>=1){
            boots++;
            bootRecharge=0;
        }

        indicateBoots();

        if(Input.GetMouseButton(0)){
            onClickWind(getClickPosition());
        }else if(Input.GetMouseButtonDown(1)){
            onClickKick(getClickPosition());
        }

        indicatePos(getClickPosition());

    }

    private void indicateBoots(){
        bootRechargeFill.fillAmount=bootRecharge;
        bootCountText.text=boots+"";
    }

    private void indicatePos(Vector3 pos_){
        cursor.transform.position=new Vector3(pos_.x,1,pos_.z);

        if(Vector3.Distance(cursor.transform.position,GameManager.Instance.BallPos)>maxWind){
            Vector3 pos2_=(cursor.transform.position-GameManager.Instance.BallPos).normalized*maxWind+GameManager.Instance.BallPos;
            cursor.transform.position=new Vector3(pos2_.x,1,pos2_.z);
        }

        cursor.transform.LookAt(GameManager.Instance.ball.ball.transform,Vector3.up);
        Vector3 e_ = cursor.transform.rotation.eulerAngles;
        cursor.transform.rotation=Quaternion.Euler(0,e_.y,0);
    }

    private Vector3 getClickPosition(){
        RaycastHit hit_;

        Ray ray_ = Camera.main.ScreenPointToRay(Input.mousePosition);
        
        if (Physics.Raycast(ray_, out hit_,1000,GameManager.GetCollisionMaskOf(worldPlane))) {
            return hit_.point;
        }

        return Vector3.zero;
    }

    private void onClickWind(Vector3 pos_){
        if(pos_==Vector3.zero){
            return;
        }
        //Debug.LogFormat("Wind {0}",pos_-GameManager.Instance.BallPos);

        wind=GameManager.Instance.BallPos-pos_;
        if(wind.magnitude>maxWind){
            wind=wind.normalized*maxWind;
        }
        windParts.transform.localPosition=Vector3.forward*10-wind.normalized*3;
        windParts.transform.LookAt(GameManager.Instance.ball.ball.transform,Vector3.up);
        var emit_=windParts.emission;
        emit_.rateOverTime=wind.magnitude*3;
        var force_=windParts.forceOverLifetime;
        force_.x=wind.x;
        force_.y=wind.y;
        force_.z=wind.z;

        
        GameManager.Instance.windySound.play();

    }

    private void onClickKick(Vector3 pos_){
        if(pos_==Vector3.zero){
            return;
        }

        if(GameManager.Instance.BallStopped || GameManager.Instance.BallMidair){
            return;
        }

        if(boots<=0){
            return;
        }

        //Debug.LogFormat("Kick {0}",pos_-GameManager.Instance.BallPos);

        kick=GameManager.Instance.BallPos-pos_;
        if(kick.magnitude>maxWind){
            kick=kick.normalized*maxWind;
        }

        
        GameManager.Instance.bootSound.play();

        GameManager.Instance.ball.spawnBoot();

        GameManager.Instance.ball.RB.AddForce(KickForce,ForceMode.Impulse);

        boots--;
        indicateBoots();

    }



}

[System.Serializable]
public class MiniSound{

    public AudioClip clip;

    public AudioClip[] clips;
    [Range(0,1)]
    public float volume;

    private AudioSource audioSource;

    public void init(Component c_){
        audioSource=c_.gameObject.GetComponent<AudioSource>();
        if(!audioSource){
            audioSource=c_.gameObject.AddComponent<AudioSource>();
        }

        audioSource.loop=false;

    }

    private AudioClip randomClip(){
        return clips[Random.Range(0,clips.Length)];
    }

    public void play(){
        if(clip==null){return;}
        if(audioSource==null){return;}
        if(clips.Length>0){
            audioSource.PlayOneShot(randomClip(),volume);
            return;
        }
        audioSource.PlayOneShot(clip,volume);
    }
}